﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion


namespace HeuristicLab.Services.Hive.Common.DataTransfer {
  public enum JobState {
    /// <summary>
    /// Job is actively calculated on a Slave
    /// </summary>
    Calculating,

    /// <summary>
    /// Job is waiting to be calculated
    /// </summary>
    Offline,

    /// <summary>
    /// Job as finished and is ready to be collected by the Client
    /// </summary>
    Finished,

    /// <summary>
    /// Job is aborted and result can be collected by the Client
    /// </summary>
    Aborted,

    /// <summary>
    /// A snapshot has been requested. The calculating slave is responsible for pausing the Job and sending the snapshot to the server. After that state will be SnapshotSent
    /// </summary>
    SnapshotRequested,

    /// <summary>
    /// Snapshot has been sent to the server and its calculating again (should'nt it be back to Calculating again?)
    /// </summary>
    SnapshotSent,

    /// <summary>
    /// ???
    /// </summary>
    Pending,

    /// <summary>
    /// Job as been aborted due to an error. Results are ready to be collected
    /// </summary>
    Failed,

    /// <summary>
    /// The job is paused and waits on the server to be sent back to a Slave when all of its child jobs are Finished.
    /// </summary>
    WaitForChildJobs
  };
}
