﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.IO;
using HeuristicLab.Services.Hive.Common.DataTransfer;
using HeuristicLab.Hive.Contracts;

namespace HeuristicLab.Services.Hive.Common.ServiceContracts {
  public interface IHiveService {

    // todo: Rename:
    //  Result is ambiguous; sometimes it refers to a serialized job which has finished, sometimes it is the current state of the job
    #region Job Methods
    [OperationContract]
    Job AddJob(JobData job);
    [OperationContract]
    Job AddJobWithGroupStrings(JobData jobObj, IEnumerable<string> groups);
    [OperationContract]
    JobData GetLastSerializedResult(Guid jobId);
    [OperationContract]
    void AbortJob(Guid jobId);
    [OperationContract]
    Job GetJobById(Guid jobId);
    [OperationContract]
    IEnumerable<LightweightJob> GetJobResults(IEnumerable<Guid> jobIds);
    [OperationContract]
    IEnumerable<LightweightJob> GetChildJobResults(Guid? parentJobId, bool recursive, bool includeParent);
    [OperationContract]
    Job AddChildJob(Guid parentJobId, JobData serializedJob);
    [OperationContract]
    Job GetAllJobsWithFilter(JobState jobState, int offset, int count);
    [OperationContract]
    Job GetAllJobs();
    [OperationContract]
    Job GetJobByIdWithDetails(Guid jobId);
    [OperationContract]
    Job AddNewJob(JobData job);
    //[OperationContract]
    //void RemoveJob(Guid jobId);
    [OperationContract]
    Job PauseJob(JobData serializedJob);
    [OperationContract]
    void DeleteChildJobs(Guid jobId);
    [OperationContract]
    Job GetJob(Guid clientId);
    [OperationContract]
    void StoreFinishedJobResult(Guid clientId, Guid jobId, byte[] result, TimeSpan executionTime, string exception);
    [OperationContract]
    void IsJobStillNeeded(Guid jobId);
    [OperationContract]
    Stream GetStreamedJob(Guid clientId);
    [OperationContract]
    void StoreFinishedJobResultStreamed(Stream stream);
    #endregion

    #region HiveExperiment Methods
    /// <summary>
    /// Returns all experiments for the current user
    /// </summary>
    [OperationContract]
    IEnumerable<HiveExperiment> GetHiveExperiments();
    [OperationContract]
    HiveExperiment UpdateHiveExperiment(HiveExperiment hiveExperimentDto);
    [OperationContract]
    void DeleteHiveExperiment(Guid hiveExperimentId);
    #endregion

    #region Project Methods
    [OperationContract]
    Project GetAllProjects();
    [OperationContract]
    void CreateProject(Project project);
    [OperationContract]
    void ChangeProject(Project project);
    [OperationContract]
    void DeleteProject(Guid projectId);
    [OperationContract]
    IEnumerable<Job> GetJobsByProject(Guid projectId);
    #endregion

    #region Login Methods
    // rename "Login"-methods to "Register" or "SayHello", since its only purpose is to send Slave-Info (is Login() should not be used anymore)
    /// <summary>
    /// Method can be used to check if security credentials are valid, but it does not do anything
    /// </summary>
    /// <returns></returns>
    [OperationContract]
    void Login();
    /// <summary>
    /// This method registers the slave and marks it as online
    /// </summary>
    [OperationContract]
    void Login(Slave slave);
    [OperationContract]
    void Logout(Guid clientId);
    #endregion

    #region Heartbeat Methods
    [OperationContract]
    List<MessageContainer> ProcessHeartBeat(HeartBeat hbData);
    #endregion

    #region Plugin Methods
    [OperationContract]
    IEnumerable<HivePluginData> GetPlugins(List<HivePlugin> pluginList);
    [OperationContract]
    Stream GetStreamedPlugins(List<HivePlugin> pluginList);
    #endregion

    #region Calendar Methods
    [OperationContract]
    IEnumerable<Appointment> GetCalendar(Guid clientId);
    [OperationContract]
    void SetCalendarStatus(Guid clientId, CalendarState state);
    [OperationContract]
    IEnumerable<Appointment> GetUptimeCalendarForResource(Guid guid);
    [OperationContract]
    void SetUptimeCalendarForResource(Guid guid, IEnumerable<Appointment> appointments, bool isForced);
    #endregion

    #region Slave Methods
    [OperationContract]
    IEnumerable<Slave> GetAllSlaves();
    [OperationContract]
    [ServiceKnownType(typeof(Resource))]
    [ServiceKnownType(typeof(Slave))]
    [ServiceKnownType(typeof(SlaveGroup))]
    IEnumerable<SlaveGroup> GetAllSlaveGroups();
    [OperationContract]
    IEnumerable<SlaveGroup> GetAllGroupsOfResource(Guid resourceId);
    [OperationContract]
    SlaveGroup AddSlaveGroup(SlaveGroup clientGroup);
    [OperationContract]
    void DeleteSlaveGroup(Guid clientGroupId);
    [OperationContract]
    [ServiceKnownType(typeof(Resource))]
    [ServiceKnownType(typeof(Slave))]
    [ServiceKnownType(typeof(SlaveGroup))]
    void AddResourceToGroup(Guid clientGroupId, Resource resource);
    [OperationContract]
    void DeleteResourceFromGroup(Guid clientGroupId, Guid resourceId);
    #endregion
    
  }
}
